@@ -1,5 +1,13 @@
 #include <Rdefines.h>
-SEXP
+#include <Rconfig.h>
+
+#ifdef HAVE_VISIBILITY_ATTRIBUTE
+# define attribute_hidden __attribute__ ((visibility ("hidden")))
+#else
+# define attribute_hidden
+#endif
+
+SEXP attribute_hidden
     lme4_DSym,
     lme4_DimSym,
     lme4_DimNamesSym,
